﻿Imports System.Runtime.CompilerServices
Imports System.ComponentModel
Imports System.Data.Common
Imports System.IO
Imports System.Text

Namespace System.Data

   Public Module DatasetX


      <Extension()> _
      Public Sub EnsureExistUnique(Of T As DataRow)(ByRef row As T, ParamArray keyCols() As DataColumn)
         Dim tbl = DirectCast(row.Table, TypedTableBase(Of T))
         Dim rw = row
         If keyCols.Length = 0 Then keyCols = tbl.PrimaryKey
         Dim keyVals = (From col In keyCols Select col, val = rw(col)).ToArray
         rw = tbl.All.FirstOrDefault(Function(rw2) keyVals.All(Function(pv) rw2(pv.col).Equals(pv.val)))
         If rw.Null Then
            tbl.Rows.Add(row)
            If keyCols.Any(Function(clm) clm.AutoIncrement) Then row.AcceptChanges()
         Else
            row.Be(rw)
         End If
      End Sub

      <Extension()> _
      Public Sub CopyTo(Of T As DataRow)(subj As T, other As T)
         Dim itms = subj.ItemArray
         Dim tb = other.Table
         For Each indx In tb.PrimaryKey.Select(Function(cl) cl.Ordinal)
            itms(indx) = other(indx)
         Next
         other.ItemArray = itms
      End Sub

      <Extension()> _
      Public Function All(Of T As DataRow)(subj As TypedTableBase(Of T), Optional filter As Func(Of T, Boolean) = Nothing) As IEnumerable(Of T)
         All = DirectCast(subj, DataTable).All.Cast(Of T)()
         If filter.NotNull Then All = All.Where(filter)
      End Function
      <Extension()> _
      Public Function All(subj As DataTable) As IEnumerable(Of DataRow)
         Return From rw In subj Where 0 = (rw.RowState And (DataRowState.Deleted Or DataRowState.Detached))
      End Function

      <Extension()> _
      Public Function ChildTables(subj As DataTable) As IEnumerable(Of DataTable)
         Return From rl In subj.ChildRelations.Cast(Of DataRelation)() Select rl.ChildTable
      End Function

      <Extension()> _
      Public Function ParentTables(subj As DataTable) As IEnumerable(Of DataTable)
         Return From rl In subj.ParentRelations.Cast(Of DataRelation)() Select rl.ParentTable
      End Function
      
      <Extension()> _
      Public Sub ClearRecursive(table As DataTable)
         Dim hsh As New HashSet(Of DataTable)
         Dim recurse As Action(Of DataTable) = _
            Sub(tb)
               If tb.Rows.Count = 0 OrElse Not hsh.Add(tb) Then Return
               tb.ChildTables.ForEach(recurse)
               tb.Clear()
            End Sub
         recurse(table)
      End Sub
      ''' <summary>topologic sort: no childtable is before its parentTable</summary>
      <Extension()> _
      Public Function GetRankedTables(dts As DataSet) As List(Of DataTable)
         Dim hsh = New HashSet(Of DataTable)(dts.Tables.Cast(Of DataTable))
         GetRankedTables = New List(Of DataTable)
         Dim recurse As Action(Of DataTable) = _
            Sub(tb)
               If Not hsh.Remove(tb) Then Return
               tb.ParentTables.ForEach(recurse)
               GetRankedTables.Add(tb)
            End Sub
         While hsh.Count > 0 : recurse(hsh.First()) : End While
      End Function

#Region "ReadWriteCompact"
      'Das Dataset wird durchlaufen, und alle Werte per TypConverter von und zu String umgewandelt. Schema-Validierung besteht darin, dass die Dataset-Struktur genau definiert, wann welcher TypConverter verwendet wird. Die Datei ist nur 1/7 so gross, v.a., weil sie keine Schema-Info enthält. Entscheidend aber, dass viel mehr Datentypen persistierbar sind als per Dataset.WriteXml - zB Fonts, Points und alle Enums - halt alles, wofür TypeConverter implementiert sind.


      Private Class DataRowConverter
         Private _KVPs As New List(Of Tuple(Of DataColumn, AllConverter)), Table As DataTable

         Public Sub New(ByVal Table As DataTable)
            Me.Table = Table
            _KVPs = ( _
               From c In Table.Columns.Cast(Of DataColumn)() Where c.Expression = "" _
               Select Tuple.Create(c, New AllConverter(c.DataType))).ToList
         End Sub
         Public Function Parse(ByVal data As String()) As DataRow
            If data.Length <> _KVPs.Count Then Throw Me.Exception("unexpected count of input-strings")
            Dim rw = Table.NewRow
            For i = 0 To _KVPs.Count - 1
               Dim s = data(i)
               With _KVPs(i)
                  rw(.Item1) = If(s = CompactReadWrite.DBNull, Convert.DBNull, .Item2.FromString(s))
               End With
            Next
            Return rw
         End Function
         Public Function Format(ByVal Row As DataRow) As String()
            Dim strings(_KVPs.Count - 1) As String
            For i = 0 To _KVPs.Count - 1
               With _KVPs(i)
                  Dim val = Row(.Item1)
                  strings(i) = If(Convert.IsDBNull(val), CompactReadWrite.DBNull, .Item2.ToString(val))
               End With
            Next
            Return strings
         End Function
      End Class

      ''' <summary>reads from file, expecting the compact-file-format</summary>
      <Extension()> _
      Public Sub LoadCompact(ByVal Subj As DataSet, ByVal _DataFile As String)
         Subj.Clear()
         Dim rankedTables = Subj.GetRankedTables.ToList
         rankedTables.ForEach(Sub(tb) tb.BeginLoadData())
         Using r = New CompactReadWrite(New StreamReader(_DataFile))
            For Each tb In rankedTables
               Dim converter = New DataRowConverter(tb)
               r.ReadTable.Select(AddressOf converter.Parse).ForEach(AddressOf tb.Rows.Add)
            Next
         End Using
         'rankedTables.ForEach(Sub(tb) tb.EndLoadData())
         For i = rankedTables.Count - 1 To 0 Step -1
            rankedTables(i).EndLoadData()
         Next
      End Sub

      ''' <summary>writes to file in compact-file-format</summary>
      <Extension()> _
      Public Sub SaveCompact(ByVal Subj As DataSet, ByVal _DataFile As String)
         Using w = New CompactReadWrite(New StreamWriter(_DataFile))
            For Each tb In Subj.GetRankedTables
               w.WriteHeader(tb.TableName, _
                  From c In tb.Columns.Cast(Of DataColumn)() Where c.Expression = "" Select c.ColumnName)
               Dim converter = New DataRowConverter(tb)
               tb.All.Select(AddressOf converter.Format).ForEach(AddressOf w.WriteRow)
               w.CloseWriteTable()
            Next
         End Using
      End Sub

#End Region 'ReadWriteCompact
   End Module

End Namespace
