﻿Imports Microsoft.VisualBasic
#If NoCompile Then
the compact-format is a primitive file-format, similar to CSV, but can store multiple tables in one file. There are 3 special-chars, reserved for structurating data: 
* _Sep: 0xf018 - at the beginning of each data-entry. 
* DBNull: 0xf019 - at the beginning of each table. Furthermore DBNull is public constant, and can be used to indicate a null-entry (and distinguish it from an empty string). 
* _NewLineMask: 0xf017 - maskes newlines within data-entries.

this is the compact-format-syntax:
a line, starting with DBNull is a tableheader. text in tableheaders will be ignored, but may improve readability
after that, lines, starting with _Sep, are dataRows.
each data-entry starts with _Sep (and the leading _Sep of a datarow belongs to its first Entry)
lines, starting with anything else than _Sep, will be ignored.
An empty line indicates the end of the table.
After a table, any line will be ignored, until the next tableheader.
#End If 'NoCompile

Imports System.Text
Namespace System.IO

   Public Class CompactReadWrite : Implements IDisposable

      Public Const DBNull As String = ChrW(&HF019)
      Private Const _Sep As Char = ChrW(&HF018)
      Private Const _NewLineMask As Char = ChrW(&HF017)
      Private Shared _Illegal As Char() = New Char() {_Sep, _NewLineMask}

      Private _LeaveOpen As Boolean, _Reader As TableReader, _Writer As TextWriter

      Public Sub New(ByVal Writer As TextWriter, Optional ByVal LeaveWriterOpen As Boolean = False)
         _LeaveOpen = LeaveWriterOpen
         _Writer = Writer
      End Sub

      Public Sub New(ByVal Reader As TextReader, Optional ByVal LeaveReaderOpen As Boolean = False)
         _LeaveOpen = LeaveReaderOpen
         _Reader = New TableReader() With {.Reader = Reader}
      End Sub

      Public Sub WriteHeader(Optional ByVal tableName As String = "table", Optional ByVal columnNames As IEnumerable(Of String) = Nothing)
         If columnNames.NotNull() Then
            tableName += ": " & JoinData(", ", columnNames)
         End If
         _Writer.WriteLine(DBNull & tableName)
      End Sub
      Public Sub WriteRow(ByVal data As IEnumerable(Of String))
         _Writer.WriteLine(_Sep & JoinData(_Sep, data))
      End Sub
      Private Function JoinData(ByVal separator As String, ByVal data As IEnumerable(Of String)) As String
         Dim sb = New StringBuilder()
         For Each s In data
            If s.IndexOfAny(_Illegal) >= 0 Then
               Throw Me.Exception("illegal chars in data")
            End If
            sb.Append(separator).Append(s.Replace(Environment.NewLine, _NewLineMask))
         Next
         Return sb.ToString(separator.Length, sb.Length - separator.Length)
      End Function

      Public Sub CloseWriteTable()
         _Writer.WriteLine()
      End Sub

      Public Function ReadTable() As IEnumerable(Of String())
         Dim s = ""
         Do 'wind forward to DBNull, which indicates Begin of next Table
            s = _Reader.Reader.ReadLine
            If s.Null Then Return New String()() {} 'end of File
         Loop Until s.StartsWith(DBNull)
         Return _Reader
      End Function

      Protected Overridable Sub Dispose(ByVal disposing As Boolean)
         If Not disposing OrElse _LeaveOpen Then Return
         If _Writer.NotNull Then
            _Writer.Dispose()
            _Writer = Nothing
         ElseIf _Reader.NotNull Then
            _Reader.Reader.Dispose()
            _Reader = Nothing
         End If
      End Sub

      Public Sub Dispose() Implements IDisposable.Dispose
         Dispose(True)
         GC.SuppressFinalize(Me)
      End Sub

      Private Class TableReader : Inherits Enumerable(Of String())
         Public Reader As TextReader

         Protected Overrides Function TryMovenext(ByRef Current() As String) As Boolean
            Dim s = ""
            Do 'wind forward to _Sep, which indicates Begin of next TableRow
               s = Reader.ReadLine.Replace(_NewLineMask, Environment.NewLine)
               If s.Length = 0 Then Return False 'end of Table
            Loop Until s(0) = _Sep
            Current = s.Substring(1).Split(_Sep)
            Return True
         End Function

      End Class

   End Class
End Namespace
