﻿#Region "FileHeader"
#If False Then
These BinarySearchers accept a selector-Delegate and a key-Value, to search Data with specific property matching to key.
Eg to search a FileInfo with specific Name in a List(Of FileInfo) you need no FileInfo with that Name, but simply can specify the searched Name.
   BinaryRangeSearcher bastardize the pure BinarySearch-Algorithm by never returning CompareResult 0 in Compare(x,y)-Method. In Effect the BinarySearch can't finish finding a match, but must search on to the final position **before** (or after) the first (last) Match. So I rapidly find the beginning of a range of matches.
#End If '-- Options, Imports
#End Region 'FileHeader

Namespace System

   Public Enum SearcherType : FirstMatch = -1 : AnyMatch : LastMatch : End Enum

   ''' <summary>Führt binäre Suchen nach einem Key aus. Bei mehreren Treffern wird der erste/letzte gewählt</summary>
   Public Class BinaryRangeComparisonSearcher(Of TElement, TProp) : Inherits Comparer(Of TElement)
      Public Key As TProp
      Public SearcherType As SearcherType = System.SearcherType.FirstMatch
      Public _Comparison As Func(Of TElement, TProp, Integer)
      Private _Match As Boolean
      Public Sub New(comparison As Func(Of TElement, TProp, Integer))
         _Comparison = comparison
      End Sub
      Public Overrides Function Compare(x As TElement, y As TElement) As Integer
         Compare = _Comparison(x, Key)
         If Compare = 0 Then _Match = True : Compare -= SearcherType
      End Function
      Public Function Search(lst As List(Of TElement), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key : _Match = False
         If count < 0 Then count = lst.Count
         Search = lst.BinarySearch(index, count, Nothing, Me)
         If _Match Then Search = CorrectResult(Search)
      End Function
      Public Function Search(lst As TElement(), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key : _Match = False
         If count < 0 Then count = lst.Length
         Search = Array.BinarySearch(lst, index, count, Nothing, Me)
         If _Match Then Search = CorrectResult(Search)
      End Function
      Private Function CorrectResult(res As Integer) As Integer
         CorrectResult = Not res
         If SearcherType = System.SearcherType.LastMatch Then Return CorrectResult - 1
      End Function
   End Class

   ''' <summary>Führt binäre Suchen nach einem Key aus. Bei mehreren Treffern wird der erste/letzte gewählt</summary>
   Public Class BinaryRangeSearcher(Of TElement, TProp As IComparable(Of TProp)) : Inherits Comparer(Of TElement)
      Private _Selector As Func(Of TElement, TProp)
      Public Key As TProp
      Public SearcherType As SearcherType
      Private _Match As Boolean
      Public Sub New(selector As Func(Of TElement, TProp))
         _Selector = selector
      End Sub
      Public Overrides Function Compare(x As TElement, y As TElement) As Integer
         Compare = _Selector(x).CompareTo(Key)
         If Compare = 0 Then _Match = True : Compare -= SearcherType
      End Function
      Public Function Search(lst As List(Of TElement), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key : _Match = False
         If count < 0 Then count = lst.Count
         Search = lst.BinarySearch(index, count, Nothing, Me)
         If _Match Then Search = CorrectResult(Search)
      End Function
      Public Function Search(lst As TElement(), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key : _Match = False
         If count < 0 Then count = lst.Length
         Search = Array.BinarySearch(lst, index, count, Nothing, Me)
         If _Match Then Search = CorrectResult(Search)
      End Function
      Private Function CorrectResult(res As Integer) As Integer
         CorrectResult = Not res
         If SearcherType = System.SearcherType.LastMatch Then Return CorrectResult - 1
      End Function
   End Class

   ''' <summary>Führt binäre Suchen nach einem Key aus</summary>
   Public Class BinarySearcher(Of TElement, TProp As IComparable(Of TProp)) : Inherits Comparer(Of TElement)
      Private _Selector As Func(Of TElement, TProp)
      Public Key As TProp
      Public Sub New(selector As Func(Of TElement, TProp))
         _Selector = selector
      End Sub
      Public Overrides Function Compare(x As TElement, y As TElement) As Integer
         Return _Selector(x).CompareTo(Key)
      End Function
      Public Overridable Function Search(lst As List(Of TElement), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key
         If count < 0 Then count = lst.Count
         Search = lst.BinarySearch(index, count, Nothing, Me)
      End Function
      Public Overridable Function Search(lst As TElement(), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key
         If count < 0 Then count = lst.Length
         Search = Array.BinarySearch(lst, index, count, Nothing, Me)
      End Function
   End Class

   ''' <summary>Führt binäre Suchen nach einem Key aus</summary>
   Public Class BinaryComparisonSearcher(Of TElement, TProp As IComparable(Of TProp)) : Inherits Comparer(Of TElement)
      Public _Comparison As Func(Of TElement, TProp, Integer)
      Public Key As TProp
      Public Sub New(comparison As Func(Of TElement, TProp, Integer))
         _Comparison = comparison
      End Sub
      Public Overrides Function Compare(x As TElement, y As TElement) As Integer
         Return _Comparison(x, Key)
      End Function
      Public Overridable Function Search(lst As List(Of TElement), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key
         If count < 0 Then count = lst.Count
         Search = lst.BinarySearch(index, count, Nothing, Me)
      End Function
      Public Overridable Function Search(lst As TElement(), key As TProp, Optional index As Integer = 0, Optional count As Integer = -1) As Integer
         Me.Key = key
         If count < 0 Then count = lst.Length
         Search = Array.BinarySearch(lst, index, count, Nothing, Me)
      End Function
   End Class

End Namespace
