﻿Imports System.Windows

Namespace System.Windows.Controls

   ''' <summary>override OnAttached to wire up the attachTarget </summary>
   Public MustInherit Class BehaviorBase(Of T As {BehaviorBase(Of T), New})
      Inherits BehaviorBaseBase(Of T, Boolean)
      'in wpf the plain Behavior-Pattern is kinda stupid: u must implement a dummi-DependencyProperty with its 4 parts. The value of the DependencyProperty isn't used anywhere - u only need the callback, when the property initially is set. This callback gives u the Object, whichs behavior u want to extend.
   End Class

   Public MustInherit Class BehaviorBase(Of T As {BehaviorBase(Of T, TProp), New}, TProp)
      Inherits BehaviorBaseBase(Of T, TProp)
      Protected _Owner As DependencyObject
      Protected Overrides Sub OnAttached(ByVal attachTarget As DependencyObject)
         _Owner = attachTarget
      End Sub
      Public Property AttachedValue As TProp
         Get
            Return GetAttachedValue(_Owner)
         End Get
         Set(ByVal value As TProp)
            SetAttachedValue(_Owner, value)
         End Set
      End Property
   End Class

   ''' <summary>
   ''' override OnAttached to wire up the attachTarget.
   ''' store attachTarget as classmember to use it for Property-access
   ''' </summary>
   Public MustInherit Class BehaviorBaseBase(Of T As {BehaviorBaseBase(Of T, TProp), New}, TProp)

      Public Shared Function GetAttachedValue(ByVal obj As DependencyObject) As TProp
         Return DirectCast(obj.GetValue(AttachedValueProperty), TProp)
      End Function
      Public Shared Sub SetAttachedValue(ByVal obj As DependencyObject, ByVal value As TProp)
         obj.SetValue(AttachedValueProperty, value)
      End Sub
      Public Shared ReadOnly AttachedValueProperty As DependencyProperty = DependencyProperty.RegisterAttached("AttachedValue", GetType(TProp), GetType(T), New UIPropertyMetadata(AddressOf AttachCallback))

      Private Shared Sub AttachCallback(ByVal sender As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
         Call (New T()).OnAttached(sender)
      End Sub
      Protected MustOverride Sub OnAttached(ByVal attachTarget As DependencyObject)
   End Class

End Namespace