﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized

Namespace System.Windows.Controls

   <System.Windows.Markup.ContentProperty("Columns")> _
   Public Class LookupConverter : Implements IValueConverter
      'usage in xaml: instantiate. add minimum 2 LookupConverterColumn, and fill them with items. first column is the key-Column, second the value-column, by default. if you add more columns, and want to convert a key to them, you must specify the column-valueColumnIndex by ConverterParameter. The filled Converter itself can be DataSource of ItemsControls

      ''' <summary> you can bind an ItemsSource direktly to the Keys in order to select from them </summary>
      Public ReadOnly Property Keys As IList(Of Object)
         Get
            If Columns.Count = 0 Then Return New Object() {}
            Return Columns(0).Items
         End Get
      End Property

      Public Property Columns As New List(Of LookupConverterColumn)

      Private Function Convert(ByVal key As Object, ByVal valueColumnIndex As Object, ByVal forward As Boolean) As Object
         Dim column = 1
         If valueColumnIndex IsNot Nothing Then Integer.TryParse(valueColumnIndex.ToString, column)
         If Not column.IsBetween(1, Columns.Count - 1) Then
            Throw Me.Exception("valueColumnIndex should match an existing Column-Index > 0")
         End If
         Dim lookIn = If(forward, Columns(0), Columns(column)).Items
         Dim returnFrom = If(forward, Columns(column), Columns(0)).Items
         Dim indx = lookIn.IndexOf(key)
         'if returnFrom contains one item more than lookIn, use it as default-value in case of noMatch
         If indx < 0 Then indx = lookIn.Count
         Return If(indx >= returnFrom.Count, Nothing, returnFrom(indx))
      End Function

      ''' <summary> retrieves a value associated with key. valueColumnIndex = 1 by default. </summary>
      Public Function Convert(ByVal key As Object, ByVal targetType As Type, ByVal valueColumnIndex As Object, _
            ByVal culture As Globalization.CultureInfo) As Object Implements Data.IValueConverter.Convert
         Return Convert(key, valueColumnIndex, True)
      End Function

      ''' <summary> retrieves the key associated with value. valueColumnIndex = 1 by default. </summary>
      Public Function ConvertBack(ByVal value As Object, ByVal targetType As Type, ByVal valueColumnIndex As Object, _
            ByVal culture As Globalization.CultureInfo) As Object Implements Data.IValueConverter.ConvertBack
         Return Convert(value, valueColumnIndex, False)
      End Function
   End Class

   <System.Windows.Markup.ContentProperty("Items")>
   Public Class LookupConverterColumn
      Public Property Items As New List(Of Object)
   End Class

End Namespace
