﻿#Region "FileHeader"
#If False Then
MainModelBase stellt eine singleton-artige shared "Instance"-Property bereit, und einen boolean Instanzen-Member "IsProvisional", welcher angibt, ob das Mainmodel nur provisorisch ist.
Vorgesehen ist, überall im Code auf den Instance-Singleton zuzugreifen, und erst zur Laufzeit in App.Xaml die einzige nicht-provisorische Instanz zu erstellen, an welche von überall gebunden werden kann.
Greift man verfrüht auf den Singleton zu, so weist er sich als IsProvisional aus, und das funzt auch im Konstruktor der abgeleiteten MainModel. Dort kann man also das Mainmodel zur Designzeit anders initialisieren als zur Laufzeit - um den Designer zu entlasten.
#End If '-- Options, Imports
#End Region 'FileHeader

Namespace System.ComponentModel

   ''' <summary>
   ''' Usage: derive any MainModel from MainModelBase(Of T) - eg: Public Class MainModel : Inherits MainModelBase(Of MainModel)
   ''' </summary>
   Public Class MainModelBase(Of T As {MainModelBase(Of T), New}) : Inherits NotifyPropertyChanged

      Private Shared _IsProvisionalStatic As Boolean
      Private Shared _Instance As T
      Public Shared ReadOnly Property Instance As T
         Get
            If _Instance Is Nothing Then
               _IsProvisionalStatic = True
               _Instance = New T
               _IsProvisionalStatic = False
            End If
            Return _Instance
         End Get
      End Property

      Public ReadOnly IsProvisional As Boolean

      Public Sub New()
         IsProvisional = _IsProvisionalStatic OrElse NotifyPropertyChanged.IsInDesignMode
         If Not (IsProvisional OrElse _Instance Is Nothing OrElse _Instance.IsProvisional) Then
            Dim msg = New List(Of Object)
            msg.Add(String.Join(GetType(T).Name, _
               "You only can instantiate ", " when ", ".Instance.IsNull or ", ".Instance.IsProvisional"))
            'msg.Add(GetType(T).Name.Between( _
            '   "You only can instantiate ", " when ", ".Instance.IsNull or ", ".Instance.IsProvisional"))
            msg.Add("IsProvisional: " & IsProvisional)
            msg.Add("Instance.IsNull: " & _Instance.Null)
            msg.Add("Designmode: " & NotifyPropertyChanged.IsInDesignMode)
            msg.Add("Process: " & Process.GetCurrentProcess().ProcessName)
            msg.Add(Process.GetCurrentProcess().ProcessName)
            If _Instance IsNot Nothing Then msg.Add("_Instance.IsProvisional: " & _Instance.IsProvisional)
            Throw New Exception(String.Join(Lf, msg))
            Throw New Exception(String.Format( _
               "you only can instantiate {0} when {0}.Instance.IsNull or {0}.Instance.IsProvisional", GetType(T).Name))
         End If
         _Instance = DirectCast(Me, T)
      End Sub

   End Class

End Namespace