﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows.Interop
Imports System.Collections

Namespace System.Windows.Data

   Public Class ChooseView(Of T) : Inherits ListCollectionView

      Private _Source As IList(Of T)
      Public Sub New(Optional sourceItems As IEnumerable(Of T) = Nothing)
         Me.New(New ObservableCollection(Of T)(If(sourceItems, New T() {})))
      End Sub
      Public Sub New(sourceList As IList(Of T))
         MyBase.New(DirectCast(sourceList, IList))
         _Source = sourceList
      End Sub

      Public ReadOnly Property Source As IList(Of T)
         Get
            Return _Source
         End Get
      End Property
      Public ReadOnly Property Current As T
         Get
            Return DirectCast(CurrentItem, T)
         End Get
      End Property
      Default Public ReadOnly Property Item(indx As Integer) As T
         Get
            Return Items(indx)
         End Get
      End Property
      Public Sub Add(itm As T)
         _Source.Add(itm)
      End Sub

      Private _Items As IList(Of T) = Nothing
      Public ReadOnly Property Items() As IList(Of T)
         Get
            If _Items Is Nothing Then
               If MyBase.SortDescriptions.Count = 0 AndAlso MyBase.Filter Is Nothing AndAlso MyBase.GroupDescriptions.Count = 0 Then
                  _Items = DirectCast(_Source, IList(Of T))
               Else
                  _Items = MyBase.Cast(Of T).ToList
               End If
            End If
            Return _Items
         End Get
      End Property

      '''' <summary>Refill SourceCollection and restore CurrentItem if possible</summary>
      '''' <param name="compareSelectors">each selector extracts a significant value from Item, to check matches with the from CurrentItem extracted value. CurrentItem is re-found, if each selector matches. Otherwise only check for Object.Equals(itm, CurrentItem)</param>
      Public Sub ReFillBy(newValues As IEnumerable(Of T), ParamArray compareSelectors() As Func(Of T, Object))
         Dim curr = Current
         If curr Is Nothing Then _Source.ReFillBy(newValues, compareSelectors) : Return
         Dim itemCheckers = compareSelectors.ToArray(Function(check As Func(Of T, Object)) New With {check, .Value = check(curr)})
         Dim selIndex = -1
         Dim callback As NotifyCollectionChangedEventHandler = _
            Sub(s, e)
               If e.Action = NotifyCollectionChangedAction.Remove Then Return
               Dim itm = DirectCast(e.NewItems(0), T)
               If Object.Equals(itm, curr) OrElse itemCheckers.All(Function(chk) Object.Equals(chk.check(itm), chk.Value)) Then
                  selIndex = e.NewStartingIndex
                  curr = itm
                  RemoveHandler Me.CollectionChanged, callback
               End If
            End Sub
         AddHandler Me.CollectionChanged, callback
         _Source.ReFillBy(newValues, compareSelectors)
         RemoveHandler Me.CollectionChanged, callback
         'empty elseif-chain cascades 3 tryals to move (best performance first)
         If Filter.Null AndAlso GroupDescriptions.Count = 0 AndAlso SortDescriptions.Count = 0 AndAlso MoveCurrentToPosition(selIndex) Then
         ElseIf MoveCurrentTo(curr) Then
         ElseIf MoveCurrentToFirst() Then
         End If
      End Sub

      Protected Overrides Sub OnCollectionChanged(args As NotifyCollectionChangedEventArgs)
         Select Case args.Action
            Case NotifyCollectionChangedAction.Reset
               _Items = Nothing 'prepare re-initialisation of _Items

               'in the following suppress an unnecessary CurrentChanged-Event by setting _Current to Null
            Case NotifyCollectionChangedAction.Add
               If CurrentPosition < 0 Then _PrevCurrent = Nothing : MoveCurrentToFirst()
            Case NotifyCollectionChangedAction.Replace
               _PrevCurrent = Nothing
            Case NotifyCollectionChangedAction.Remove
               If Count > 0 Then _PrevCurrent = Nothing
         End Select
         MyBase.OnCollectionChanged(args)
      End Sub

      Private _PrevCurrent As Object = Nothing
      Public ReadOnly Property PrevCurrent As T
         Get
            Return DirectCast(_PrevCurrent, T)
         End Get
      End Property
      Protected Overrides Sub OnCurrentChanged()
         If Object.Equals(_PrevCurrent, CurrentItem) Then Return
         MyBase.OnCurrentChanged()
         _PrevCurrent = CurrentItem
      End Sub
   End Class

End Namespace