﻿Imports System.Collections.Generic
Imports System.Runtime.CompilerServices
Imports System.ComponentModel
Imports System.Windows.Controls.Primitives
Imports System.Windows.Media

Namespace System.Windows

   Public Module ControlsX

      Public ReadOnly ThicknessEmpty As New Thickness(0)

      <Extension> _
      Public Function TransformBounds(tr As GeneralTransform, sz As Size) As Rect
         Return tr.TransformBounds(New Rect(sz))
      End Function
      <Extension> _
      Public Function TransformBounds(tr As GeneralTransform, x As Double, y As Double, width As Double, height As Double) As Rect
         Return tr.TransformBounds(New Rect(x, y, width, height))
      End Function

      ''' <summary> bitwise state of pressed mousbuttons - bit-meanings: 1=Left, 2=Middle, 4=Right </summary>
      <Extension> _
      Public Function GetMouseButtonState(e As MouseEventArgs) As Integer
         Return e.LeftButton Or e.MiddleButton << 1 Or e.RightButton << 2
      End Function

      <Extension()>
      Public Sub MakeSeparators(grd As DataGrid)
         Dim p = grd.FindChild(Of DataGridColumnHeadersPresenter)()
         Dim cp = p.FindChild(Of DataGridCellsPanel)()
         For Each child As DataGridColumnHeader In cp.Children
            With child
               .SeparatorVisibility = Windows.Visibility.Visible
               If .SeparatorBrush.Null Then .SeparatorBrush = Brushes.Black
               If .Padding.Equals(ThicknessEmpty) Then .Padding = New Thickness(0.5, 0, 0.5, 0)
            End With
         Next
      End Sub

      <Extension(), DebuggerStepThrough()>
      Public Function MoveFocusX(itm As UIElement, direction As FocusNavigationDirection, Optional wrapped As Boolean = False) As Boolean
         Return itm.MoveFocus(New TraversalRequest(direction) With {.Wrapped = wrapped})
      End Function
      <Extension(), DebuggerStepThrough()>
      Public Function MoveFocusX(itm As ContentElement, direction As FocusNavigationDirection, Optional wrapped As Boolean = False) As Boolean
         Return itm.MoveFocus(New TraversalRequest(direction) With {.Wrapped = wrapped})
      End Function

      <Extension(), DebuggerStepThrough()>
      Public Function SizeSize(rct As Rect) As Double
         With rct
            If .Width < 0 AndAlso .Height < 0 Then Return -.Width * .Height
            Return .Width * .Height
         End With
      End Function

      <Extension(), DebuggerStepThrough()>
      Public Function OriginAs(Of T)(e As RoutedEventArgs) As T
         Return DirectCast(e.OriginalSource, T)
      End Function
      Private Function ItemOrCurrent(Of T As Class)(itm As Object) As T
         With TryCast(itm, ICollectionView)
            If .NotNull Then Return TryCast(.CurrentItem, T)
         End With
         Return TryCast(itm, T)
      End Function
      <Extension(), DebuggerStepThrough()>
      Public Function OriginContextAs(Of T As Class)(e As RoutedEventArgs) As T
         Dim origin = e.OriginalSource
         With TryCast(origin, FrameworkContentElement)
            If .Null Then Return TryCast(DirectCast(origin, FrameworkElement).DataContext, T)
            Return TryCast(.DataContext, T)
            Return ItemOrCurrent(Of T)(.DataContext)
         End With
      End Function
      <Extension(), DebuggerStepThrough()>
      Public Function ContextAs(Of T As Class)(dpo As DependencyObject) As T
         With TryCast(dpo, FrameworkContentElement)
            If .Null Then Return TryCast(DirectCast(dpo, FrameworkElement).DataContext, T)
            Return ItemOrCurrent(Of T)(.DataContext)
         End With
      End Function

      '<Extension(), DebuggerStepThrough()>
      'Public Function ContextAs(Of T)(dpo As DependencyObject) As T
      '   With TryCast(dpo, FrameworkContentElement)
      '      If .Null Then Return DirectCast(DirectCast(dpo, FrameworkElement).DataContext, T)
      '      Return ItemOrCurrent(Of T)(.DataContext)
      '   End With
      'End Function

      <Extension(), DebuggerStepThrough()>
      Public Function ContentAs(Of T)(ctl As ContentControl) As T
         Return DirectCast(ctl.Content, T)
      End Function

      <Extension()>
      Public Function LocationOnScreen(fe As Visual) As Point
         Return fe.PointToScreen(New Point)
      End Function
      <Extension()>
      Public Function RectOnScreen(fe As FrameworkElement) As Rect
         Return New Rect(fe.PointToScreen(New Point), fe.RenderSize)
      End Function

      <Extension(), DebuggerStepThrough()>
      Public Function FindResourceX(Of T)(fe As FrameworkElement, key As String) As T
         Dim ret = DirectCast(fe.TryFindResource(key), T)
         If ret Is Nothing Then Stop
         Return ret
      End Function
      <Extension()>
      Public Function TryFindResourceX(Of T)(fe As FrameworkElement, key As String) As T
         Return DirectCast(fe.TryFindResource(key), T)
      End Function

      <Extension(), DebuggerStepThrough()>
      Public Function SelectionEnd(Of T As TextBox)(tb As T) As Integer
         Return tb.SelectionLength + tb.SelectionStart
      End Function

      <Extension()>
      Public Sub EnableOthers(except As DependencyObject, enable As Boolean)
         Dim parent = except
         Do
            Dim obj = VisualTreeHelper.GetParent(parent)
            If obj.Null Then Exit Do
            parent = obj
         Loop
         Dim dumpOn As Func(Of DependencyObject, Boolean) = Nothing
         dumpOn = Function(dpd)
                     If dpd Is except Then Return True
                     Dim enableRange As Action(Of Integer, Integer) = _
                        Sub(lbnd, ubnd)
                           For ii = lbnd To ubnd
                              With TryCast(VisualTreeHelper.GetChild(dpd, ii), UIElement)
                                 If .NotNull Then .IsEnabled = enable
                              End With
                           Next
                        End Sub
                     Dim ubound = VisualTreeHelper.GetChildrenCount(dpd) - 1
                     For i = 0 To ubound
                        If dumpOn(VisualTreeHelper.GetChild(dpd, i)) Then
                           enableRange(0, i - 1)
                           enableRange(i + 1, ubound)
                           Return True
                        End If
                     Next
                     Return False
                  End Function
         dumpOn(parent)
      End Sub

   End Module

End Namespace