﻿Imports System.Windows.Documents
Imports System.Windows.Media
Imports System.Windows.Threading
Imports System.Threading

Namespace System.Windows.Controls

   Public Enum DropStrategy : DropOn = 2 : DropOnOrAfterlast : Between : BetweenOrAfterlast : End Enum
   Public Class DragOverAdorner : Inherits Adorner
      Public DropStrategy As DropStrategy = Controls.DropStrategy.DropOn

      Public Class EventArgs : Inherits System.EventArgs
         Public ReadOnly DrawingContext As DrawingContext
         Public ReadOnly Rect As Rect
         Public Handled As Boolean = False
         Friend Sub New(DrawingContext As DrawingContext, Rect As Rect)
            Me.DrawingContext = DrawingContext : Me.Rect = Rect
         End Sub
      End Class
      Public Event Render As EventHandler(Of EventArgs)
      Public Event TimeOut As EventHandler

      Private _Dispatcher As Dispatcher = Dispatcher.CurrentDispatcher
      Private _Timeout As New Timer(Sub() _Dispatcher.BeginInvoke(OnTimeout))
      Private OnTimeout As Action = Sub()
                                       RaiseEvent TimeOut(Me, EventArgs.Empty)
                                       Rect = Rect.Empty
                                    End Sub

      Private ReadOnly _ItemsControl As ItemsControl

      Public Sub New(adorned As UIElement)
         MyBase.New(adorned)
         _ItemsControl = DirectCast(adorned, ItemsControl)
         adorned.AllowDrop = True
         IsHitTestVisible = False
      End Sub

      Private _Rect As Rect = Rect.Empty
      Public Property Rect As Rect
         Get
            Return _Rect
         End Get
         Set(value As Rect)
            _Timeout.Change(150, System.Threading.Timeout.Infinite)
            If _Rect = value Then Return
            If value.IsEmpty Then
               AdornerLayer.GetAdornerLayer(AdornedElement).Remove(Me)
            ElseIf _Rect.IsEmpty Then
               AdornerLayer.GetAdornerLayer(AdornedElement).Add(Me)
            Else
               InvalidateVisual()
            End If
            _Rect = value
         End Set
      End Property

      Public Sub Refresh(itemContainer As FrameworkElement, dragArgs As DragEventArgs)
         Dim rct = Rect.Empty
         Try
            If dragArgs.Effects = DragDropEffects.None Then Return
            If itemContainer.Null Then
               If 0 = (DropStrategy And 1) Then Return
               Dim indx = _ItemsControl.Items.Count
               If indx = 0 Then
                  rct = New Rect(0, 0, _ItemsControl.ActualWidth, 0)
               Else
                  Dim itm = _ItemsControl.ItemContainerGenerator.ContainerFromIndex(indx - 1)
                  itemContainer = DirectCast(itm, FrameworkElement)
                  Dim tr = itemContainer.TransformToAncestor(_ItemsControl)
                  rct = tr.TransformBounds(0, itemContainer.ActualHeight, itemContainer.ActualWidth, 0)
                  If DropStrategy.HasFlag(DropStrategy.Between) Then rct = MakeBetweenLine(rct, dragArgs)
                  If dragArgs.GetPosition(_ItemsControl).Y < rct.Bottom Then rct = Rect.Empty
               End If
            Else
               Dim tr = itemContainer.TransformToAncestor(_ItemsControl)
               rct = tr.TransformBounds(itemContainer.RenderSize)
               If DropStrategy.HasFlag(DropStrategy.Between) Then rct = MakeBetweenLine(rct, dragArgs)
            End If
         Finally
            If rct.IsEmpty Then dragArgs.Effects = DragDropEffects.None
            Rect = rct
         End Try
      End Sub

      Private Function MakeBetweenLine(rct As Rect, dragArgs As DragEventArgs) As Rect
         If 0 = (DropStrategy.Between And DropStrategy) Then Return rct
         Dim halfHeight = rct.Height / 2
         If dragArgs.GetPosition(_ItemsControl).Y > rct.Y + halfHeight Then rct.Y += rct.Height
         rct.Height = 0
         Return rct
      End Function

      Protected Overrides Sub OnRender(drawingContext As Windows.Media.DrawingContext)
         If _Rect.IsEmpty Then Return
         Dim e = New EventArgs(drawingContext, _Rect)
         RaiseEvent Render(Me, e)
         If e.Handled Then Return
         Dim pen = New Pen(Brushes.Red, 4)
         drawingContext.DrawRoundedRectangle(Nothing, pen, e.Rect, 2, 2)
      End Sub
   End Class

End Namespace
